# Product Context: slacraper

## プロジェクトの存在理由

Slack は多くの組織やチームで使用されているコミュニケーションツールですが、過去のメッセージを特定の条件で検索・抽出し、プログラムで処理するための簡単なツールが不足しています。slacraper は、この問題を解決するために開発されました。

## 解決する問題

1. **メッセージの自動収集**

   - 特定のチャンネルから定期的にメッセージを収集し、分析や保存が必要な場合
   - 重要な情報を自動的にアーカイブする場合
   - チームの活動履歴を記録する場合

2. **特定条件のメッセージ抽出**

   - 特定のユーザーのメッセージのみを抽出したい場合
   - 特定のキーワードを含むメッセージを抽出したい場合
   - 特定の時間範囲内のメッセージを抽出したい場合

3. **リアクション分析**

   - 特定のリアクション（スタンプ）が付いたメッセージを収集したい場合
   - 人気のあるコンテンツを分析したい場合
   - チーム内でのエンゲージメントを測定したい場合

4. **自動化プロセスの一部**
   - 他のスクリプトやワークフローの一部として Slack メッセージを処理したい場合
   - CI/CD パイプラインの一部として通知内容を取得したい場合
   - 定期的なレポート生成の一部としてメッセージを集計したい場合

## 機能要件

### 必須機能（実装済み）

1. **チャンネル指定**

   - 特定のチャンネルからメッセージを取得する
   - チャンネル名から自動的にチャンネル ID を解決する

2. **認証**

   - Slack Bot Token を使用して API 認証を行う
   - 環境変数または引数からトークンを取得する

3. **時間範囲指定**

   - 指定した時間範囲内のメッセージを取得する（デフォルト: 1 時間以内）
   - 時間単位で柔軟に指定可能

4. **フィルタリング**

   - チャンネル名（必須）: 特定のチャンネルに限定
   - 投稿ユーザー名: 特定のユーザーのメッセージのみを抽出
   - メッセージ文字列の内容: 特定のキーワードを含むメッセージのみを抽出
   - 付与されたリアクション（スタンプ）: 特定のリアクションが付いたメッセージのみを抽出

5. **構造化出力**

   - JSON 形式での出力
   - 一貫した構造でプログラムでの処理が容易

6. **ライブラリ API**

   - Python プログラムから利用可能な API
   - 柔軟なパラメータ設定
   - 型ヒントによる使いやすさ

7. **CLI インターフェース**
   - コマンドラインからの利用
   - ヘルプメッセージによる使い方の説明
   - エラー時の適切なフィードバック

### オプション機能（実装済み）

1. **メッセージ URL**
   - Slack メッセージへの直接リンク URL を出力に含める
   - ブラウザで直接メッセージを開くことが可能

### 将来的な機能（未実装）

1. **ページネーション対応**

   - 大量のメッセージを取得する場合の自動ページネーション
   - メモリ効率の良い処理

2. **出力形式のカスタマイズ**

   - CSV, YAML など他の形式での出力オプション
   - カスタム出力フォーマットの定義

3. **スレッド返信の取得**

   - メッセージのスレッド返信も含めて取得するオプション
   - スレッド構造の保持

4. **ファイル添付の情報取得**

   - メッセージに添付されたファイルの情報も取得するオプション
   - ファイルのダウンロードリンクの提供

5. **並列処理**
   - 複数チャンネルからの同時取得
   - パフォーマンスの向上

## ユーザー体験の目標

1. **シンプルさ**

   - 最小限のセットアップで使用できる
   - 直感的なインターフェース
   - 明確なドキュメント

2. **柔軟性**

   - ライブラリとしても CLI としても使用可能
   - 様々なユースケースに対応
   - カスタマイズ可能なオプション

3. **信頼性**

   - エラー処理が適切
   - 予期しない状況でも適切なフィードバックを提供
   - 安定した動作

4. **拡張性**
   - 将来的な機能追加が容易な設計
   - モジュール化されたコード構造
   - 明確な API 設計

## ユースケース例

1. **チーム活動の記録**

   ```python
   # 毎日のチーム活動を記録
   from slacraper import SlackScraper
   import json
   import datetime

   scraper = SlackScraper(channel="team-updates")
   messages = scraper.get_messages(hours=24)

   # 日付ごとにファイルに保存
   today = datetime.date.today().isoformat()
   with open(f"team-updates-{today}.json", "w") as f:
       json.dump(messages, f, indent=2)
   ```

2. **重要なアナウンスの抽出**

   ```bash
   # 「重要」というキーワードを含むメッセージを抽出
   slacraper --channel announcements --text-contains "重要" --include-url
   ```

3. **人気コンテンツの分析**

   ```python
   # 特定のリアクションが付いたメッセージを分析
   from slacraper import SlackScraper

   scraper = SlackScraper(channel="ideas")
   popular_ideas = scraper.get_messages(
       hours=168,  # 過去1週間
       reaction="thumbsup",
       include_url=True
   )

   # リアクション数でソート
   sorted_ideas = sorted(
       popular_ideas,
       key=lambda msg: sum(r["count"] for r in msg.get("reactions", []))
   )

   # 上位5件を表示
   for idea in sorted_ideas[-5:]:
       print(f"{idea['user_name']}: {idea['text']}")
       print(f"URL: {idea['url']}")
       print("---")
   ```

4. **CI/CD 通知の監視**
   ```bash
   # CI/CDの成功/失敗通知を抽出してレポート
   slacraper --channel ci-cd-notifications --hours 24 --text-contains "failed" > failed_builds.json
   ```
